package gogo

import (
	"bytes"
	"errors"
	"fmt"
	"os"
	"os/exec"
)

const (
	goDirName = "go"
)

// GoConfig - Env variables for Go compiler
type GoConfig struct {
	DLLName string
	GOARCH  string
}

// GoCmd - Execute a go command
func GoCmd(config GoConfig, cwd string) ([]byte, error) {
	//goBinPath := path.Join(config.GOROOT, "bin", "go")
	//docker run --rm -it -v /root/cgo:/go/work \
	// -w /go/work \
	// x1unix/go-mingw go build .
	dockerimage := "gogo:v1"
	var goarch []string
	if config.GOARCH == "x86" {
		goarch = []string{"-e", "GOARCH=386", "-e", "CC=i686-w64-mingw32-gcc"}
	} else if config.GOARCH == "x64" {
		goarch = []string{"-e", "GOARCH=amd64", "-e", "CC=x86_64-w64-mingw32-gcc"}
	} else {
		return nil, errors.New("arch not support")
	}
	var command = []string{
		"run",
		"--rm",
		"-i",
		"-v",
		fmt.Sprintf("%s:%s", cwd, "/go/work"),
		"-w",
		"/go/work",
		"-e",
		"LDFLAGS=\"-s -w\"",
		"-e",
		"CGO_ENABLED=1",
	}
	command = append(command, goarch...)
	command = append(command, []string{dockerimage, "/bin/sh", "-c"}...)
	goCommand := fmt.Sprintf("go mod init test && go build -ldflags \"-s -w\" -o %s -buildmode c-shared", config.DLLName)
	command = append(command, goCommand)
	cmd := exec.Command("docker", command...)
	cmd.Dir = cwd
	cmd.Env = os.Environ()
	var stdout bytes.Buffer
	var stderr bytes.Buffer
	cmd.Stdout = &stdout
	cmd.Stderr = &stderr

	fmt.Printf("go cmd: '%v'\n", cmd)
	err := cmd.Run()
	if err != nil {
		fmt.Printf("--- env ---\n")
		for _, envVar := range cmd.Env {
			fmt.Printf("%s\n", envVar)
		}
		fmt.Printf("--- stdout ---\n%s\n", stdout.String())
		fmt.Printf("--- stderr ---\n%s\n", stderr.String())
	}

	return stdout.Bytes(), err
}
